﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Transforms
{

    [Api(Description = "今日流水操作统计")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentAcceptStatistic:RestfulExecution<Payment>
    {
        public DateTime BeginTime { get; set; }//日期筛选为创建日期,默认统计今日导入系统中的流水信息

        public DateTime EndTime { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
            if (BeginTime == DateTime.MinValue) BeginTime = DateTime.Now;
            if (EndTime == DateTime.MinValue) EndTime = DateTime.Now;

            var beginTime = DateTime.Parse(BeginTime.ToString("yyyy-MM-dd"));
            var endTime = DateTime.Parse(EndTime.ToString("yyyy-MM-dd"));
            if (endTime < beginTime) throw new Exception("查询结束日期不可小于开始日期!");
            if (endTime == beginTime) endTime = endTime.AddDays(1);
            var conn = KiviiContext.GetOpenedDbConnection<Payment>();
            var queryAccount = conn.From<Account>();
            queryAccount.Where(o => o.OrganizationKvid == KiviiContext.CurrentMember.OrganizationKvid && Sql.In(o.Type, AccountType.Balance, AccountType.Cash, AccountType.Pos));
            queryAccount.Select(o => o.Kvid);

            var sqlExpress = conn.From<Payment>();
            sqlExpress.Where(o => o.RootKvid == o.Kvid && o.OffsetKvid == Guid.Empty && Sql.In(o.Type, PaymentType.AliPay, PaymentType.WeChat, PaymentType.Bank, PaymentType.Cash, PaymentType.Pos));
            sqlExpress.And(o => o.CreateTime >= beginTime && o.CreateTime < endTime);
            //sqlExpress.And(o => o.AmountSplited == 0 || o.AmountSplited < o.Amount);
            sqlExpress.And(o => (Sql.In(o.PayerAccountKvid, queryAccount)));
            sqlExpress.Select(o => new { o.AmountSplited, o.AmountInvoice, o.AmountUsed, o.Amount });
            var results = conn.Select(sqlExpress);

            var rtns = new StatisticResponse<Payment>();
            //rtns.Description = "待认领流水";
            rtns.BeginTime = BeginTime;
            rtns.EndTime = EndTime;
            if (results.IsNullOrEmpty()) return rtns;
            rtns.Total = results.Count();
            rtns.TotalAmount = results.Sum(o => o.Amount);
            rtns.TotalAmountSplited = results.Sum(o => o.Amount - o.AmountSplited);
            rtns.TotalQuantitySplited = results.Where(o => o.AmountSplited == 0 || o.AmountSplited < o.Amount).Count();
            rtns.TotalAmountInvoice = results.Sum(o => o.Amount - o.AmountInvoice);
            rtns.TotalQuantityInvoice = results.Where(o => o.AmountInvoice == 0 || o.AmountInvoice < o.Amount).Count();
            rtns.TotalAmountUsed = results.Sum(o => o.Amount - o.AmountUsed);
            rtns.TotalQuantityUsed = results.Where(o => o.AmountUsed == 0 || o.AmountUsed < o.Amount).Count();
            return rtns;
        }
    }

    [Api(Description = "已认领流水分析")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class PaymentSplitedAnalysis : RestfulExecution<Payment>
    {
        public Guid OwnerKvid { get; set; }
        public List<Guid> OwnerKvids { get; set; }

        public bool GroupByOwner { get; set; }

        public DateTime BeginTime { get; set; }

        public DateTime EndTime { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
            if (BeginTime == DateTime.MinValue) BeginTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
            if (EndTime == DateTime.MinValue) EndTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, monthDay);

            var beginTime = DateTime.Parse(BeginTime.ToString("yyyy-MM-dd"));
            var endTime = DateTime.Parse(EndTime.ToString("yyyy-MM-dd"));
            if (endTime < beginTime) throw new Exception("查询结束日期不可小于开始日期!");
            if (endTime == beginTime) endTime = endTime.AddDays(1);
            if (OwnerKvids.IsNullOrEmpty()) OwnerKvids = new List<Guid>();
            if (OwnerKvid != Guid.Empty) OwnerKvids.Add(OwnerKvid);

            var conn = KiviiContext.GetOpenedDbConnection<Payment>();
            var queryAccount = conn.From<Account>();
            queryAccount.Where(o => o.OrganizationKvid == KiviiContext.CurrentMember.OrganizationKvid && Sql.In(o.Type, AccountType.Balance, AccountType.Cash, AccountType.Pos));
            queryAccount.Select(o => o.Kvid);

            var sqlExpress = conn.From<Payment>();
            sqlExpress.Where(o => o.OffsetKvid == Guid.Empty && Sql.In(o.Type, PaymentType.WeChat, PaymentType.Cash, PaymentType.AliPay, PaymentType.Pos, PaymentType.Split, PaymentType.Refund, PaymentType.UnBiz));
            sqlExpress.And(o => o.OperateTime >= beginTime && o.OperateTime < endTime);
            if (!OwnerKvids.IsNullOrEmpty()) sqlExpress.And(o => Sql.In(o.OwnerKvid, OwnerKvids));
            sqlExpress.And(o => (Sql.In(o.PayerAccountKvid, queryAccount)));
            sqlExpress.OrderBy(o => o.OwnerName);
            sqlExpress.Select(o => new { o.OwnerKvid, o.OwnerName, o.Amount, o.AmountInvoice, o.AmountUsed, o.OperateTime, o.CreateTime, o.Type });
            var results = conn.Select(sqlExpress);
            var rtns = new StatisticResponse<Analysis>();
            if (results.IsNullOrEmpty()) return rtns;
            rtns.Results = new List<Analysis>();
            rtns.BeginTime = BeginTime;
            rtns.EndTime = EndTime;

            //传入部门信息就按照部门group
            if (GroupByOwner)
            {
                var group = results.GroupBy(o => o.OwnerName);
                foreach (var kv in group)
                {
                    var rtn = new Analysis();
                    rtn.Summary = kv.Key;
                    rtn.Amount = kv.Sum(o => o.Amount);
                    rtn.Quantity = kv.Count();
                    rtn.AmountSplit = kv.Where(o => o.Type == PaymentType.Split || o.Type == PaymentType.WeChat || o.Type == PaymentType.Cash || o.Type == PaymentType.AliPay || o.Type == PaymentType.Pos).Sum(o => o.Amount);
                    rtn.QuantitySplit = kv.Count(o => o.Type == PaymentType.Split || o.Type == PaymentType.WeChat || o.Type == PaymentType.Cash || o.Type == PaymentType.AliPay || o.Type == PaymentType.Pos);
                    rtn.AmountInvoice = kv.Where(o => o.Type == PaymentType.Split || o.Type == PaymentType.WeChat || o.Type == PaymentType.Cash || o.Type == PaymentType.AliPay || o.Type == PaymentType.Pos).Sum(o => o.AmountInvoice);
                    rtn.QuantityInvoice = kv.Where(o => o.Type == PaymentType.Split || o.Type == PaymentType.WeChat || o.Type == PaymentType.Cash || o.Type == PaymentType.AliPay || o.Type == PaymentType.Pos).Count(o => o.Amount == o.AmountInvoice);
                    rtn.AmountNotInvoice = rtn.AmountSplit - rtn.AmountInvoice;
                    rtn.QuantityNotInvoice = rtn.QuantitySplit - rtn.QuantityInvoice;
                    rtn.AmountUsed = kv.Where(o => o.Type == PaymentType.Split || o.Type == PaymentType.WeChat || o.Type == PaymentType.Cash || o.Type == PaymentType.AliPay || o.Type == PaymentType.Pos).Sum(o => o.AmountUsed);
                    rtn.QuantityUsed = kv.Where(o => o.Type == PaymentType.Split || o.Type == PaymentType.WeChat || o.Type == PaymentType.Cash || o.Type == PaymentType.AliPay || o.Type == PaymentType.Pos).Count(o => o.Amount == o.AmountUsed);
                    rtn.AmountNotUsed = rtn.AmountSplit - rtn.AmountUsed;
                    rtn.QuantityNotUsed = rtn.QuantitySplit - rtn.QuantityUsed;
                    rtn.AmountRefund = kv.Where(o => o.Type == PaymentType.Refund).Sum(o => o.Amount);
                    rtn.QuantityRefund = kv.Count(o => o.Type == PaymentType.Refund);
                    rtn.AmountUnBiz = kv.Where(o => o.Type == PaymentType.UnBiz).Sum(o => o.Amount);
                    rtn.QuantityUnBiz = kv.Count(o => o.Type == PaymentType.UnBiz);
                    rtns.Results.Add(rtn);
                }
            }
            else
            {
                var rtn = new Analysis();
                rtn.Amount = results.Sum(o => o.Amount);
                rtn.Quantity = results.Count();
                rtn.AmountInvoice = results.Where(o => o.Type == PaymentType.Split || o.Type == PaymentType.WeChat || o.Type == PaymentType.Cash || o.Type == PaymentType.AliPay || o.Type == PaymentType.Pos).Sum(o => o.AmountInvoice);
                rtn.QuantityInvoice = results.Where(o => o.Type == PaymentType.Split || o.Type == PaymentType.WeChat || o.Type == PaymentType.Cash || o.Type == PaymentType.AliPay || o.Type == PaymentType.Pos).Count(o => o.Amount == o.AmountInvoice);
                rtn.AmountUsed = results.Where(o => o.Type == PaymentType.Split || o.Type == PaymentType.WeChat || o.Type == PaymentType.Cash || o.Type == PaymentType.AliPay || o.Type == PaymentType.Pos).Sum(o => o.AmountUsed);
                rtn.QuantityUsed = results.Where(o => o.Type == PaymentType.Split || o.Type == PaymentType.WeChat || o.Type == PaymentType.Cash || o.Type == PaymentType.AliPay || o.Type == PaymentType.Pos).Count(o => o.Amount == o.AmountUsed);
                rtn.AmountSplit = results.Where(o => o.Type == PaymentType.Split || o.Type == PaymentType.WeChat || o.Type == PaymentType.Cash || o.Type == PaymentType.AliPay || o.Type == PaymentType.Pos).Sum(o => o.Amount);
                rtn.QuantitySplit = results.Count(o => o.Type == PaymentType.Split || o.Type == PaymentType.WeChat || o.Type == PaymentType.Cash || o.Type == PaymentType.AliPay || o.Type == PaymentType.Pos);
                rtn.AmountRefund = results.Where(o => o.Type == PaymentType.Refund).Sum(o => o.Amount);
                rtn.QuantityRefund = results.Count(o => o.Type == PaymentType.Refund);
                rtn.AmountUnBiz = results.Where(o => o.Type == PaymentType.UnBiz).Sum(o => o.Amount);
                rtn.QuantityUnBiz = results.Count(o => o.Type == PaymentType.UnBiz);
                rtns.Results.Add(rtn);
            }
            return rtns;
        }
    }
}
